struct VertexToPixel
{
	float4 Position     : POSITION;
	float4 Color        : COLOR0;
	float2 ScreenCoord  : TEXCOORD0;
};

struct PixelToFrame
{
    float4 Color : COLOR0;
};


float Width;
float Height;

int Length;
uniform texture DataTexture;
bool Toggle;
int ScrollOffset;
int Scale;
int DisplayColor;

sampler textureSampler  = sampler_state
{
	Texture				= <DataTexture>;
	magfilter			= POINT;	// magfilter - bigger than actual
	minfilter			= POINT;	// minfilter - smaller than actual
	mipfilter			= POINT;//LINEAR;
};



VertexToPixel SimplestVertexShader( float4 inPos : POSITION, float4 inColor : COLOR0, float2 inUV : TEXCOORD0)
{
	VertexToPixel Output = (VertexToPixel)0;
	Output.Position = inPos; //mul(inPos, xViewProjection);
	Output.Color.rgb = inColor.rgb;
	Output.ScreenCoord.xy = inUV.xy;

	return Output;    
}

float convertToCoord(float uv_value, float length, float scale)
{
	float offset = 0;
	if (length == 16 || length == 32 || length == 32 || length == 64 || length == 128 || length == 256 || length == 512 || length == 1024)
		offset = -1;
	float value = floor(round(uv_value * length + offset) / scale);
	return value;
}

PixelToFrame OurFirstPixelShader(VertexToPixel PSIn)
{
	if (!Toggle)
	{
		const float texture_size = 4096.0;
		PixelToFrame Output = (PixelToFrame)0;

		float x = convertToCoord(PSIn.ScreenCoord.x, Width, Scale);
		float y = convertToCoord(PSIn.ScreenCoord.y, Height, Scale);
	
		//convert UV/Screen Coordinates to array positons
		int screenPos = x + y * (Width / Scale) + ScrollOffset;
		int step = screenPos % 4;
		int intPos = screenPos / 4;

		if (screenPos < Length)
		{
			float gray = 0.5;

			//convert array position to texture reads
			float2 dataUV;
			dataUV.x = (intPos % texture_size) / texture_size;
			dataUV.y = (intPos /  (int)texture_size) / texture_size;

			float4 color = tex2D(textureSampler, dataUV);

			if (step==0) gray = color.b;
			else if (step==1) gray = color.g;
			else if (step==2) gray = color.r;
			else gray = color.a;
		
			if (DisplayColor == 0)//Grayscale
			{
				Output.Color.r = gray;
				Output.Color.g = gray;
				Output.Color.b = gray;
				Output.Color.a = 1.0;
			}
			else if (DisplayColor == 1)//False Color
			{
				Output.Color.r = Output.Color.g = Output.Color.b = 0;
				Output.Color.a = 1;

				if (gray < 0.25)
				{
					Output.Color.r = gray * 4;
				}
				else if (gray < 0.50)
				{
					Output.Color.r = 2 - gray * 4;
					Output.Color.g = gray * 4 - 1;
				}
				else if (gray < 0.75)
				{
					Output.Color.g = 3 - gray * 4;
					Output.Color.b = gray * 4 - 2;
				}
				else
				{
					Output.Color.r = gray * 4 - 3;
					Output.Color.g = gray * 4 - 3;
					Output.Color.b = 1;
				}
			}
			else if (DisplayColor == 2)//Extended False Color
			{
				Output.Color.r = Output.Color.g = Output.Color.b = 0;
				Output.Color.a = 1;
				
				float gray256 = gray * 256;

				Output.Color.r = (gray * 16) % 1.001;
				Output.Color.g = (gray * 4) % 1.001;
				Output.Color.b = gray;
			}
		}
		else
		{
			Output.Color.r = (x) % 2;
			Output.Color.g = (y) % 2;
			Output.Color.b = 1 - (x + y) % 2;
		}

		return Output;
	}
	else
	{
		PixelToFrame Output = (PixelToFrame)0;
		Output.Color = tex2D(textureSampler, PSIn.ScreenCoord);
		return Output;
	}
}


technique Simplest
{
	pass Pass0
	{
		VertexShader = compile vs_3_0 SimplestVertexShader();
		PixelShader = compile ps_3_0 OurFirstPixelShader();
	}
}
